package cz.davidak.ftpcam;

import android.content.Context;
import android.content.SharedPreferences;
import android.os.AsyncTask;
import android.os.Environment;
import android.preference.PreferenceManager;
import android.util.Log;

import cz.davidak.ftpcam.FtpLog;

import org.apache.commons.net.ftp.FTPClient;
import org.apache.commons.net.ftp.FTPConnectionClosedException;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.net.SocketException;
import java.net.UnknownHostException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * Created by Administrator on 10.2.2015.
 */
class AsyncTaskFTP extends AsyncTask<ArrayList<Object>, Void, Boolean> {
    private Context context;
    private Exception exception;
    private static final String TAG = "AsyncTaskFTP";
    //private ArrayList<Object>[] input;

    @Override
    protected Boolean doInBackground(ArrayList<Object>... input) {
        SharedPreferences SP = PreferenceManager.getDefaultSharedPreferences(context);
        boolean ftp_on = SP.getBoolean("pref_ftp_on", false);
        boolean http_on = SP.getBoolean("pref_http_on", false);
        String fileName = SP.getString("pref_file_name", "zk.jpg");
        String timeLapsePath = SP.getString("pref_time_lapse_path", "timelapse");
        boolean timeLapseFile = SP.getBoolean("pref_time_lapse_file", false);

        String dirTimeStampName = new SimpleDateFormat("dd-MM-yyyy").format(new Date());
        String fileTimeStampName = new SimpleDateFormat("HH-mm-ss").format(new Date());
        String timeLapseFileName = fileTimeStampName + ".jpg";

        if(ftp_on) { //ftp uploading
            FTPClient client = (FTPClient) input[0].get(1);
            try {
                FileInputStream fis = new FileInputStream((String) input[0].get(0));
                FileInputStream fis2 = new FileInputStream(FtpLog.getLogPath());
                FileInputStream fisTLf = new FileInputStream((String) input[0].get(0));

                String ip = SP.getString("pref_ip", "domafranc.tk");
                int port = Integer.parseInt(SP.getString("pref_port", "21"));
                int mode = Integer.parseInt(SP.getString("pref_mode", "0"));
                String username = SP.getString("pref_username", "pictures");
                String password = SP.getString("pref_password", "pictures");
                String logFileName = SP.getString("pref_log_ftp", "app.log");

                boolean loginInfo = false;
                if (!client.isConnected()) {
                    client.connect(ip, port);
                    Log.e(TAG, "FTP connecting...");
                    loginInfo = client.login(username, password);
                } else {
                    Log.e(TAG, "FTP already connected");
                    loginInfo = true;
                }
                Log.e(TAG, "FTP connected, sending NoOp...");
                boolean noopStatus = client.sendNoOp(); //sending NOOP and testing for response
                if (noopStatus) {
                    Log.e(TAG, "FTP NoOp test OK");
                } else {
                    Log.e(TAG, "FTP NoOp test FAILED --> reconnecting!");
                    client.disconnect();
                    client.connect(ip, port);
                    loginInfo = client.login(username, password);
                }
                Log.e(TAG, "FTP login: " + loginInfo);
                if (loginInfo) {
                    client.setFileType(FTPClient.BINARY_FILE_TYPE);
                    if (mode == 0)
                        client.enterLocalActiveMode();
                    else if (mode == 1)
                        client.enterLocalPassiveMode();
                    boolean status = client.storeFile("new_" + fileName, fis);
                    if (!status) {
                        SharedPreferences.Editor editor = SP.edit();
                        if (mode == 0) {
                            Log.e(TAG, "FTP probably active mode not allowed, trying passive");
                            client.enterLocalPassiveMode();
                            editor.putString("pref_mode", "1");
                            status = client.storeFile("new_" + fileName, fis);
                            Log.e(TAG, "Entered in PASIVE mode, file status: " + status);
                        } else if (mode == 1) {
                            Log.e(TAG, "FTP probably passive mode not allowed, trying active");
                            client.enterLocalActiveMode();
                            editor.putString("pref_mode", "0");
                            status = client.storeFile("new_" + fileName, fis);
                            Log.e(TAG, "Entered in ACTIVE mode, file status: " + status);
                        }
                        editor.commit();
                    }
                    boolean statusTLfile = false;
                    boolean statusTLdir = false;
                    boolean statusLog = client.storeFile(logFileName, fis2);
                    if (timeLapseFile) {
                        client.makeDirectory(timeLapsePath);
                        statusTLdir = client.changeWorkingDirectory(timeLapsePath);
                        client.makeDirectory(dirTimeStampName);
                        statusTLdir = client.changeWorkingDirectory(dirTimeStampName);
                        statusTLfile = client.storeFile(timeLapseFileName, fisTLf);
                        client.changeWorkingDirectory("/");
                    }
                    int replyCode = client.getReplyCode();
                    String replyString = client.getReplyString();
                    Log.e(TAG, "FTP storeFile1: " + status + " | storeFileLog: " + statusLog + " | ReplyCode: " + replyCode + " | ReplyString: " + replyString);
                    if (timeLapseFile)
                        Log.e(TAG, "FTP storeFile3: " + statusTLfile + " | changeDir: " + statusTLdir);
                    status = client.deleteFile(fileName);
                    Log.e(TAG, "FTP deleteFile: " + status);
                    status = client.rename("new_" + fileName, fileName);
                    Log.e(TAG, "FTP renameFile: " + status);
                    //FtpLog.putLog("FTP-file1: " + status + " | file2: " + statusLog);
                } else {
                    Log.e(TAG, "FTP login ERR - try another account!");
                }
                return true;
            } catch (FTPConnectionClosedException e) {
                Log.e(TAG, "FTP Connection ERR: " + e.getMessage());
                try {
                    Log.e(TAG, "Disconnecting!");
                    client.disconnect();
                } catch (IOException err) {
                    Log.e(TAG, "FTP Disconnect ERR: " + err.getMessage());
                }
                return false;
            } catch (SocketException e) {
                Log.e(TAG, "FTP Socket ERR: " + e.getMessage());
                try {
                    Log.e(TAG, "Disconnecting!");
                    client.disconnect();
                } catch (IOException err) {
                    Log.e(TAG, "FTP Disconnect ERR: " + err.getMessage());
                }
                return false;
            } catch (UnknownHostException e) {
                Log.e(TAG, "FTP Unknown host ERR: " + e.getMessage());
                try {
                    Log.e(TAG, "Disconnecting!");
                    client.disconnect();
                } catch (IOException err) {
                    Log.e(TAG, "FTP Disconnect ERR: " + err.getMessage());
                }
                return false;
            } catch (Exception e) {
                e.printStackTrace();
                return false;
            }
        }
        else if (http_on) //http uploading
        {
            Log.e(TAG, "HTTP enabled --> uploading...");
            String url = SP.getString("pref_http_url", "http://domafranc.planashift.cz");
            String param_name = SP.getString("pref_http_param_name", "");
            String param_val = SP.getString("pref_http_param_val", "");

            try {
                File file = new File((String) input[0].get(0));
                MultipartUtility multipart = new MultipartUtility(url, "UTF-8");
                if(param_name != "")
                {
                    multipart.addFormField(param_name, param_val);
                }
                multipart.addFormField("file_name", fileName);
                if(timeLapseFile)
                {
                    multipart.addFormField("timelapse_path", timeLapsePath + "/" + dirTimeStampName);
                    multipart.addFormField("timelapse_file_name", timeLapseFileName);
                }
                multipart.addFilePart("file", file);
                List<String> response = multipart.finish(); // response from server.
                Log.e(TAG, "HTTP file uploading done - response: " + response);

                return true;
            } catch (SocketException e) {
                Log.e(TAG, "HTTP Socket ERR: " + e.getMessage());
                return false;
            } catch (UnknownHostException e) {
                Log.e(TAG, "HTTP Unknown host ERR: " + e.getMessage());
                return false;
            } catch (IOException e) {
                Log.e(TAG, "HTTP IO ERR: " + e.getMessage());
                return false;
            } catch (Exception e) {
                e.printStackTrace();
                return false;
            }
        }
        return true;
    }

    public void setContext(Context context){
        this.context = context;
    }
    protected void onPostExecute(Boolean result) {
        Log.d(TAG, "vysledek: " + result);
    }

}
