package cz.davidak.ftpcam;

import android.app.Notification;
import android.app.PendingIntent;
import android.app.Service;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Matrix;
import android.graphics.Paint;

import android.graphics.Rect;

import android.graphics.SurfaceTexture;
import android.hardware.Camera;

import android.os.Build;
import android.os.Environment;
import android.os.Handler;
import android.os.IBinder;
import android.os.PowerManager;
import android.preference.ListPreference;
import android.preference.PreferenceManager;
import android.support.v4.app.NotificationCompat;
import android.util.Log;
import android.view.SurfaceHolder;
import android.widget.Toast;

import org.apache.commons.net.ftp.FTPClient;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Timer;
import java.util.TimerTask;

public class CaptureService extends Service {
    public static Camera mServiceCamera;
    private static final String TAG = "CaptureService";
    public static final int MEDIA_TYPE_IMAGE = 1;
    public static final int MEDIA_TYPE_VIDEO = 2;
    public static TimerTask mTimerTask;
    static Timer t = new Timer();
    static final Handler handler = new Handler();
    private static String path;
    public static SurfaceHolder surfaceHolder;
    private static Context context;
    private static Context baseContext;
    private PowerManager.WakeLock wakeLock;
    private Notification notify;
    public static float degrees = 0;
    public static int camera_type = 1;
    public static ListPreference pictureResolution;
    private static FTPClient ftpClient = new FTPClient();

    public CaptureService() {
    }

    @Override
    public IBinder onBind(Intent intent) {
        // TODO: Return the communication channel to the service.
        throw new UnsupportedOperationException("Not yet implemented");
    }

    @Override
    public int onStartCommand(Intent intent, int flags, int startId) {
        return START_STICKY_COMPATIBILITY;
    }
    @Override
    public void onCreate() {
        super.onCreate();

        Intent intent = new Intent(this, MainActivity.class);
        PendingIntent pIntent = PendingIntent.getActivity(this, (int) System.currentTimeMillis(), intent, PendingIntent.FLAG_UPDATE_CURRENT);
        if (Build.VERSION.SDK_INT >= 16) {
            notify = new Notification.Builder(this)
                    .setContentTitle("FTP CAM service Active!")
                    .setContentText("Application service running!")
                    .setSmallIcon(R.mipmap.ic_launcher)
                    .setContentIntent(pIntent)
                    .build();
        } else {
            notify = new NotificationCompat.Builder(this)
                    .setContentTitle("FTP CAM service Active!")
                    .setContentText("Application service running!")
                    .setSmallIcon(R.mipmap.ic_launcher)
                    .setContentIntent(pIntent)
                    .build();
        }
        startForeground(1, notify);

        context = getApplicationContext();
        baseContext = getBaseContext();
        SharedPreferences SP = PreferenceManager.getDefaultSharedPreferences(getBaseContext());
        camera_type = Integer.parseInt(SP.getString("pref_camera_type", "1"));
        if (camera_type == 0) {
            mServiceCamera = getCameraInstance(Camera.CameraInfo.CAMERA_FACING_FRONT);
        } else {
            mServiceCamera = getCameraInstance(Camera.CameraInfo.CAMERA_FACING_BACK);
        }
        pictureResolution = new ListPreference(this);
        Camera.Parameters camPar = mServiceCamera.getParameters();
        List<Camera.Size> sizes = camPar.getSupportedPictureSizes();
        CharSequence[] charSequence = new String[sizes.size()];
        int i = 0;
        for (Camera.Size size : sizes)
        {
            charSequence[i] = size.width + "x" + size.height;
            i++;
        }
        pictureResolution.setEntries(charSequence);
        pictureResolution.setEntryValues(charSequence);
        String pictRes = SP.getString("pref_picture_resolution", "640x480");
        int width = Integer.parseInt(pictRes.split("x")[0]);
        int height = Integer.parseInt(pictRes.split("x")[1]);
        camPar.setPictureSize(width, height);
        mServiceCamera.setParameters(camPar);

        PowerManager mgr = (PowerManager)context.getSystemService(context.POWER_SERVICE);
        wakeLock = mgr.newWakeLock(PowerManager.PARTIAL_WAKE_LOCK, "CaptureServiceWakeLock");
        wakeLock.acquire();

        try {
            degrees = SP.getFloat("display_rotation", 90);
            if(MainActivity.created)
            {
                if (degrees == 0.0) {
                    //param.setPreviewSize(height, width);
                    CaptureService.mServiceCamera.setDisplayOrientation(90);
                }
                if (degrees == 90.0) {
                    //param.setPreviewSize(width, height);
                    CaptureService.mServiceCamera.setDisplayOrientation(0);
                }

                if (degrees == 180.0) {
                    //param.setPreviewSize(height, width);
                    CaptureService.mServiceCamera.setDisplayOrientation(270);
                }

                if (degrees == 270.0) {
                    //param.setPreviewSize(width, height);
                    CaptureService.mServiceCamera.setDisplayOrientation(180);
                }
                surfaceHolder = MainActivity.surfaceHolder;
                mServiceCamera.setPreviewDisplay(surfaceHolder);
                Log.e("CAP_SERVICE", "Holder for MainActivity created");
            }
            else
            {
                if(Build.VERSION.SDK_INT >= 16) {
                    SurfaceTexture surfaceTexture = new SurfaceTexture(10);
                    mServiceCamera.setPreviewTexture(surfaceTexture);
                }
                //SurfaceView surfaceView = new SurfaceView(getBaseContext());
                //surfaceHolder = surfaceView.getHolder();
                Log.e("CAP_SERVICE", "Texture for service created");
            }
            mServiceCamera.startPreview();
            mServiceCamera.autoFocus(null);
        } catch (IOException e) {
            Log.e(TAG, "Error setting CAMERA preview: " + e.getMessage());
        }

        boolean serviceON = SP.getBoolean("service_on", false);
        if (serviceON) {
            Toast.makeText(getApplicationContext(), "TIMER ON",
                    Toast.LENGTH_SHORT).show();
            doTimerTask();
            Log.e("SERVICE", "Timer ON");
        }
        FtpLog.putLog("CAP_SERVICE: Service ON", getApplicationContext());
        Toast.makeText(getApplicationContext(), "Service ON",
                Toast.LENGTH_SHORT).show();
    }
    public void onDestroy() {
        super.onDestroy();
        wakeLock.release();
        stopTask();
        releaseCamera();
        try {
            ftpClient.logout();
            ftpClient.disconnect();
        } catch (Exception e) {

        }
        FtpLog.putLog("CAP_SERVICE: Service OFF", getApplicationContext());
        Toast.makeText(getApplicationContext(), "Service OFF",
                Toast.LENGTH_SHORT).show();
    }
    public static void takePicture() {
            mServiceCamera.takePicture(null, null, mPicture);
        Log.e("SERVICE", "Before Take Pic");
    }
    public static void autoFocus() {
        mServiceCamera.autoFocus(null);
    }
    public static void doTimerTask() {

        mTimerTask = new TimerTask() {
            public void run() {
                handler.post(new Runnable() {
                    public void run() {
                        takePicture();
                    }
                });
            }
        };
        Toast.makeText(context, "Auto foto ON!", Toast.LENGTH_SHORT).show();
        SharedPreferences SP = PreferenceManager.getDefaultSharedPreferences(baseContext);
        String intervalString = SP.getString("pref_interval", "60");
        int timerInterval = Integer.parseInt(intervalString);

        if (timerInterval > 4) {
            // public void schedule (TimerTask task, long delay, long period)
            t.schedule(mTimerTask, 0, timerInterval * 1000);
        } else {
            t.schedule(mTimerTask, 0, 60000);
        }

    }

    public static void stopTask() {

        if (mTimerTask != null) {
            mTimerTask.cancel();
            Toast.makeText(context, "Auto foto OFF!", Toast.LENGTH_SHORT).show();;
        }

    }

    public static Camera getCameraInstance(int cameraId) {
        Camera c = null;
        try {
            Log.e("CAP_SERVICE", "CAMERA Type: " + cameraId);
            c = Camera.open(cameraId); // attempt to get a Camera instance
        } catch (Exception e) {
            // Camera is not available (in use or does not exist)
        }
        return c; // returns null if camera is unavailable
    }

    private void releaseCamera(){
        if (mServiceCamera != null){
            mServiceCamera.release();        // release the camera for other applications
            mServiceCamera = null;
        }
    }

    public static Camera.PictureCallback mPicture = new Camera.PictureCallback() {

        @Override
        public void onPictureTaken(byte[] data, Camera camera) {

            Log.e("SERVICE", "On pic taken");

            String timeStamp = new SimpleDateFormat("dd.MM.yyy  HH:mm:ss").format(new Date());
            Paint paint = new Paint();
            paint.setTextSize(12);
            paint.setColor(Color.WHITE);
            paint.setStyle(Paint.Style.FILL);

            SharedPreferences SP = PreferenceManager.getDefaultSharedPreferences(baseContext);
            int width = Integer.parseInt(SP.getString("pref_picture_width", "320"));
            int height = Integer.parseInt(SP.getString("pref_picture_height", "320"));
            int quality = SP.getInt("pref_picture_quality", 100);

            BitmapFactory.Options BFOptions = new BitmapFactory.Options();
            //BFOptions.inSampleSize = 4;
            Bitmap bitmap = BitmapFactory.decodeByteArray(data, 0, data.length, BFOptions);
            //Bitmap resizedBitmap = Bitmap.createScaledBitmap(bitmap, width, height, true);
            //bitmap.recycle();
            Matrix m = new Matrix();
            int bitmapWidth = bitmap.getWidth();
            int bitmapHeight = bitmap.getHeight();
            float scaleWidth = ((float) width) / bitmapWidth;
            float scaleHeight = ((float) height) / bitmapHeight;
            m.postScale(scaleWidth,scaleHeight);
            if(degrees == 90.0) m.postRotate(degrees - 90);
            else if(degrees == 0.0) {
                if(camera_type == 1) m.postRotate(degrees + 90);
                else m.postRotate(degrees - 90);
            }
            else if(degrees == 270.0) m.postRotate(degrees - 90);
            else m.postRotate(degrees + 90);
            Bitmap resizedBitmap = Bitmap.createBitmap(bitmap, 0, 0, bitmapWidth, bitmapHeight, m, true);
            bitmap.recycle();
            Canvas canvas = new Canvas(resizedBitmap);
            Rect textBounds = new Rect();

            paint.getTextBounds(timeStamp, 0, timeStamp.length(), textBounds); //zjištění šířky textu
            Rect backRect = new Rect(
                    resizedBitmap.getWidth()-textBounds.width()-2,
                    resizedBitmap.getHeight()-textBounds.height()-2,
                    resizedBitmap.getWidth(),
                    resizedBitmap.getHeight());
            canvas.drawRect(backRect, paint); //pozadí
            paint.setColor(Color.RED);
            canvas.drawText(timeStamp, (float) resizedBitmap.getWidth() - textBounds.width()-2,(float)resizedBitmap.getHeight()-2, paint); //vypsání textu
            ByteArrayOutputStream blob = new ByteArrayOutputStream();
            resizedBitmap.compress(Bitmap.CompressFormat.JPEG, quality, blob);
            File pictureFile = getOutputMediaFile(MEDIA_TYPE_IMAGE);
            resizedBitmap.recycle();
            if (pictureFile == null) {
                Log.d(TAG, "Error creating media file, check storage permissions");
                return;
            }
            mServiceCamera.startPreview();
            try {
                FileOutputStream fos = new FileOutputStream(pictureFile);

                fos.write(blob.toByteArray());
                //fos.write(data);
                fos.close();

                AsyncTaskFTP ftpTask = new AsyncTaskFTP();
                ftpTask.setContext(baseContext);
                ArrayList<Object> arrayList = new ArrayList<Object>();
                arrayList.add(0, path);
                arrayList.add(1, ftpClient);
                ftpTask.execute(arrayList);
                //mCamera = getCameraInstance();

            } catch (FileNotFoundException e) {
                Log.d(TAG, "File not found: " + e.getMessage());
            } catch (IOException e) {
                Log.d(TAG, "Error accessing file: " + e.getMessage());
            }
        }
    };

    private static File getOutputMediaFile(int type) {
        // To be safe, you should check that the SDCard is mounted
        // using Environment.getExternalStorageState() before doing this.

        File mediaStorageDir = new File(Environment.getExternalStoragePublicDirectory(
                Environment.DIRECTORY_PICTURES), "FTPCam");
        path = mediaStorageDir.getPath() + File.separator +
                "foto.jpg";
        // This location works best if you want the created images to be shared
        // between applications and persist after your app has been uninstalled.

        // Create the storage directory if it does not exist
        if (!mediaStorageDir.exists()) {
            if (!mediaStorageDir.mkdirs()) {
                Log.d("MyCameraApp", "failed to create directory");
                return null;
            }
        }

        // Create a media file name
        String timeStamp = new SimpleDateFormat("yyyyMMdd_HHmmss").format(new Date());
        File mediaFile;
        if (type == MEDIA_TYPE_IMAGE) {
            mediaFile = new File(path); //jen jedna fotka
        } else if (type == MEDIA_TYPE_VIDEO) {
            mediaFile = new File(mediaStorageDir.getPath() + File.separator +
                    "VID_" + timeStamp + ".mp4");
        } else {
            return null;
        }

        return mediaFile;
    }
}
