package cz.davidak.ftpcam;

import java.lang.String;
import java.util.ArrayList;
import java.util.List;

import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.PixelFormat;
import android.graphics.PorterDuff;
import android.graphics.Rect;
import android.hardware.Camera;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.net.wifi.WifiConfiguration;
import android.net.wifi.WifiManager;
import android.os.Build;
import android.preference.PreferenceManager;
import android.support.v7.app.ActionBarActivity;
import android.os.Bundle;
import android.util.Log;
import android.view.Display;
import android.view.Menu;
import android.view.MenuItem;
import android.view.MotionEvent;
import android.view.Surface;
import android.view.SurfaceHolder;
import android.view.SurfaceView;
import android.view.View;
import android.view.View.OnTouchListener;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.CompoundButton;
//import android.widget.Switch;
import android.widget.Toast;



public class MainActivity extends ActionBarActivity implements SurfaceHolder.Callback{

    public static SurfaceHolder surfaceHolder;
    public static SurfaceView surfaceView;
    public static SurfaceView transparentSW;
    public static SurfaceHolder transparentSH;
    public static Display display;
    private static final String TAG = "MainActivity";
    private CheckBox switchAuto;
    private Intent serviceIntent;
    public static boolean created = false;
    public int surfacesCreatedCount = 0;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        FtpLog.putLog("APP: Started", getBaseContext());

        created = true;

        //wifiConnect();

        serviceIntent = new Intent(MainActivity.this, CaptureService.class);
        serviceIntent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);

        surfaceView = new SurfaceView(getBaseContext());
        surfaceView = (SurfaceView) findViewById(R.id.cameraSurfaceView);
        surfaceHolder = surfaceView.getHolder();
        surfaceHolder.addCallback(MainActivity.this);
        surfaceHolder.setType(SurfaceHolder.SURFACE_TYPE_PUSH_BUFFERS);

        //transparentSW = new SurfaceView(getBaseContext());
        transparentSW = (SurfaceView) findViewById(R.id.transparentSurfaceView);
        transparentSH = transparentSW.getHolder();
        transparentSH.setFormat(PixelFormat.TRANSPARENT);
        transparentSH.addCallback(MainActivity.this);
        transparentSH.setType(SurfaceHolder.SURFACE_TYPE_PUSH_BUFFERS);

        display = getWindowManager().getDefaultDisplay();
        switchAuto = (CheckBox) findViewById(R.id.switch_auto);

        SharedPreferences SP = PreferenceManager.getDefaultSharedPreferences(getBaseContext());

        boolean serviceON = SP.getBoolean("service_on", false);
        switchAuto.setChecked(serviceON);

        transparentSW.setOnTouchListener(new OnTouchListener() {
            @Override
            public boolean onTouch(View v, MotionEvent event) {
                if (event.getAction() == MotionEvent.ACTION_DOWN) {
                    if (Build.VERSION.SDK_INT >= 14) {
                        float x = event.getX();
                        float y = event.getY();
                        Log.e("MainActivity - Touch", "Touched! X: " + event.getX() + " Y: " + event.getY());
                        Canvas canvas = transparentSH.lockCanvas();
                        if (canvas != null) {
                            canvas.drawColor(0, PorterDuff.Mode.CLEAR);
                            Paint paint = new Paint();
                            paint.setStyle(Paint.Style.STROKE);
                            paint.setColor(Color.RED);
                            paint.setStrokeWidth(3);
                            canvas.drawRect(x - 50, y - 50, x + 50, y + 50, paint);
                            transparentSH.unlockCanvasAndPost(canvas);
                        }
                        else
                        {
                            Log.e("MainActivity - Touch", "Canvas is null!");
                        }
                        if (CaptureService.mServiceCamera != null) {
                            CaptureService.mServiceCamera.cancelAutoFocus();
                            Camera.Parameters parameters = CaptureService.mServiceCamera.getParameters();

                            Log.e("MainActivity - Touch", "maxFocus: " + parameters.getMaxNumFocusAreas() + " focusModes: " + parameters.getSupportedFocusModes());
                            if (parameters.getMaxNumFocusAreas() > 0) {
                                Log.e("MainActivity - Touch", "Count of Focus Areas is bigger than 0! That's good.");
                                int focusX = Math.round((x-surfaceView.getWidth()/2) * (1000/(surfaceView.getWidth()/2)));
                                int focusY = Math.round((y-surfaceView.getHeight()/2) * (1000/(surfaceView.getHeight()/2)));
                                int rectHalf = 50;
                                if(focusX+rectHalf >= 1000) focusX = focusX - ((focusX+rectHalf)-999);
                                if(focusY+rectHalf >= 1000) focusY = focusY - ((focusY+rectHalf)-999);
                                if(focusX-rectHalf <= -1000) focusX = focusX - ((focusX-rectHalf)+999);
                                if(focusY-rectHalf <= -1000) focusY = focusY - ((focusY-rectHalf)+999);
                                Rect rect = new Rect(focusX-50, focusY-50, focusX+50, focusY+50);//calculateFocusArea(event.getX(), event.getY());


                                parameters.setFocusMode(Camera.Parameters.FOCUS_MODE_AUTO);
                                List<Camera.Area> meteringAreas = new ArrayList<Camera.Area>(1);
                                meteringAreas.add(new Camera.Area(rect, 800));
                                parameters.setFocusAreas(meteringAreas);

                                CaptureService.mServiceCamera.setParameters(parameters);
                                CaptureService.mServiceCamera.autoFocus(null);

                                List<Camera.Area> camAreas = parameters.getFocusAreas();
                                Rect camRect = null;
                                int weight = 0;
                                for(Camera.Area camA : camAreas)
                                {
                                    camRect = camA.rect;
                                    weight = camA.weight;
                                }
                                Log.e("MainActivity - Touch", "Rect! top: " + camRect.top + " left: " + camRect.left + " bottom: " + camRect.bottom + " right: " + camRect.right + " weight: " + weight);
                            } else {
                                Log.e("MainActivity - Touch", "Count of Metering Areas is less than 0! That's bad.");
                                CaptureService.mServiceCamera.autoFocus(null);
                            }
                        } else {
                            Log.e("MainActivity - Touch", "CaptureServiceCamera is null bro...");
                        }
                    }
                }
                return true;
            }
        });
        Button captureButton = (Button) findViewById(R.id.button_capture);
        captureButton.setOnClickListener(
                new View.OnClickListener() {
                    @Override
                    public void onClick(View v) {
                        // get an image from the camera
                        CaptureService.takePicture();
                        FtpLog.putLog("APP: Manual photo taken", getBaseContext());
                    }
                }
        );
        Button autoFocusButton = (Button) findViewById(R.id.buttonAutoFocus);
        autoFocusButton.setOnClickListener(
                new View.OnClickListener() {
                    @Override
                    public void onClick(View v) {
                        CaptureService.autoFocus();
                    }
                }
        );
        Button showSettings = (Button) findViewById(R.id.showSettings);
        showSettings.setOnClickListener(
                new View.OnClickListener() {
                    @Override
                    public void onClick(View v) {
                        // get an image from the camera

                        SharedPreferences SP = PreferenceManager.getDefaultSharedPreferences(getBaseContext());
                        String intervalString = SP.getString("pref_interval", "60");
                        int quality = SP.getInt("pref_picture_quality", 100);

                        int interval = Integer.parseInt(intervalString);
                        boolean serviceON = SP.getBoolean("service_on", false);
                        int mode = Integer.parseInt(SP.getString("pref_mode", "0"));
                        Toast.makeText(getApplicationContext(), "Mode: " + mode + " | Quality: " + quality + " | Interval: " + interval + " | ServiceON: "+ serviceON, Toast.LENGTH_LONG).show();
                    }
                }
        );
        switchAuto.setOnCheckedChangeListener(new CompoundButton.OnCheckedChangeListener() {
            @Override
            public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
                SharedPreferences SP = PreferenceManager.getDefaultSharedPreferences(getBaseContext());
                SharedPreferences.Editor editor = SP.edit();
                if (isChecked) {
                    editor.putBoolean("service_on", true);
                    editor.commit();
                    CaptureService.doTimerTask();
                    FtpLog.putLog("APP: AutoPhoto turned ON", getBaseContext());
                } else {
                    CaptureService.stopTask();
                    editor.putBoolean("service_on", false);
                    editor.commit();
                    FtpLog.putLog("APP: AutoPhoto turned OFF", getBaseContext());
                }
            }
        });
    }

    @Override
    protected void onPause() {

        super.onPause();
        SharedPreferences SP = PreferenceManager.getDefaultSharedPreferences(getBaseContext());
        SharedPreferences.Editor editor = SP.edit();
        boolean serviceON = SP.getBoolean("service_on", false);
        boolean backgroundON = SP.getBoolean("pref_run_background", false);
        if(!serviceON || !backgroundON)
        {
            switchAuto.setChecked(false);
            editor.putBoolean("service_on", false);
            if(!backgroundON)
            {
                editor.putBoolean("pref_run_on_start", false);
            }
            editor.commit();
            stopService(new Intent(MainActivity.this, CaptureService.class)); //neukončí se hned
        }
    }

    @Override
    protected void onStop() {
        super.onStop();
        FtpLog.putLog("APP: Stopped", getBaseContext());
    }

    protected void onResume() {
        super.onResume();
        Log.e("MainActivity", "Activity resumed!");
        SharedPreferences SP = PreferenceManager.getDefaultSharedPreferences(getBaseContext());
        SharedPreferences.Editor editor = SP.edit();
        float degrees = 0;
        if (display.getRotation() == Surface.ROTATION_0) {
            degrees = 0;
        }
        if (display.getRotation() == Surface.ROTATION_90) {
            degrees = 90;
        }
        if (display.getRotation() == Surface.ROTATION_180) {
            degrees = 180;
        }
        if (display.getRotation() == Surface.ROTATION_270) {
            degrees = 270;
        }
        editor.putFloat("display_rotation", degrees);
        editor.commit();
        //boolean serviceON = SP.getBoolean("service_on", false);
        //boolean backgroundON = SP.getBoolean("pref_run_background", false);
        //if(!(serviceON && backgroundON)) startService(serviceIntent);

        //dle nastavení zapnutí služby a když ne, tak přiřazení surfaceview
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        // Inflate the menu; this adds items to the action bar if it is present.
        getMenuInflater().inflate(R.menu.menu_main, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        // Handle action bar item clicks here. The action bar will
        // automatically handle clicks on the Home/Up button, so long
        // as you specify a parent activity in AndroidManifest.xml.
        int id = item.getItemId();

        //noinspection SimplifiableIfStatement
        if (id == R.id.action_settings) {
            Intent intent = new Intent(this, SettingsActivity.class);
            intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
            startActivity(intent);
            return true;
        }

        return super.onOptionsItemSelected(item);
    }
    @Override
    public void surfaceCreated(SurfaceHolder holder) {
        if (holder.equals(transparentSH)) {
            surfacesCreatedCount++;
            Log.e("MainActivity", "transparentSurf created");
        } else if (holder.equals(surfaceHolder)) {
            surfacesCreatedCount++;
            Log.e("MainActivity", "surfView created");
        } else {
            surfacesCreatedCount++;
            Log.e("MainActivity", "Some different surface holder created");
        }
        if(surfacesCreatedCount >= 4)
        {
            //protože surfaces musí být načteny/vytvořeny dřív než spuštaná service - problém s náhledem jinak
            Log.e("MainActivity", "All surfaces done - going to start service");
            SharedPreferences SP = PreferenceManager.getDefaultSharedPreferences(getBaseContext());
            boolean serviceON = SP.getBoolean("service_on", false);
            boolean backgroundON = SP.getBoolean("pref_run_background", false);
            if (!(serviceON && backgroundON)) startService(serviceIntent);
            surfacesCreatedCount = 0;

            try {
                if (CaptureService.mServiceCamera != null)
                {
                    CaptureService.mServiceCamera.stopPreview();

                    //CaptureService.mServiceCamera.setParameters(param);
                    CaptureService.mServiceCamera.setPreviewDisplay(holder);
                    CaptureService.mServiceCamera.startPreview();
                }
            } catch (Exception e)
            {

            }
        }
    }

    @Override
    public void surfaceChanged(SurfaceHolder holder, int format, int width, int height) {
        if (holder.equals(transparentSH)) {
            surfacesCreatedCount++;
            Log.e("MainActivity", "transparentSurf changed");
        } else if (holder.equals(surfaceHolder)) {
            surfacesCreatedCount++;
            Log.e("MainActivity", "surfView changed");
        } else {
            surfacesCreatedCount++;
            Log.e("MainActivity", "Some different surface holder changed");
        }
        if(surfacesCreatedCount >= 4)
        {
            Log.e("MainActivity", "All surfaces done - going to start service");
            SharedPreferences SP = PreferenceManager.getDefaultSharedPreferences(getBaseContext());
            boolean serviceON = SP.getBoolean("service_on", false);
            boolean backgroundON = SP.getBoolean("pref_run_background", false);
            if (!(serviceON && backgroundON)) startService(serviceIntent);
            surfacesCreatedCount = 0;

            try {
                if (CaptureService.mServiceCamera != null)
                {
                    CaptureService.mServiceCamera.stopPreview();

                    //CaptureService.mServiceCamera.setParameters(param);
                    CaptureService.mServiceCamera.setPreviewDisplay(holder);
                    CaptureService.mServiceCamera.startPreview();
                }
            } catch (Exception e)
            {

            }
        }
        try {
            /*CaptureService.mServiceCamera.stopPreview();
            Camera.Parameters param = CaptureService.mServiceCamera.getParameters();
            Display display = getWindowManager().getDefaultDisplay();
            if (display.getRotation() == Surface.ROTATION_0) {
                param.setPreviewSize(height, width);
                CaptureService.mServiceCamera.setDisplayOrientation(90);
            }
            if (display.getRotation() == Surface.ROTATION_90) {
                param.setPreviewSize(width, height);
                CaptureService.mServiceCamera.setDisplayOrientation(0);
            }

            if (display.getRotation() == Surface.ROTATION_180) {
                param.setPreviewSize(height, width);
                CaptureService.mServiceCamera.setDisplayOrientation(270);
            }

            if (display.getRotation() == Surface.ROTATION_270) {
                param.setPreviewSize(width, height);
                CaptureService.mServiceCamera.setDisplayOrientation(180);
            }
            CaptureService.mServiceCamera.setParameters(param);
            CaptureService.mServiceCamera.setPreviewDisplay(holder);
            CaptureService.mServiceCamera.startPreview(); */
        } catch (Exception e)
        {

        }
    }

    @Override
    public void surfaceDestroyed(SurfaceHolder holder) {
        // TODO Auto-generated method stub

    }
    public void wifiConnect() {
        ConnectivityManager cm = (ConnectivityManager)getApplicationContext().getSystemService(Context.CONNECTIVITY_SERVICE);
        NetworkInfo activeNetwork = cm.getActiveNetworkInfo();
        if(activeNetwork == null) {
            String ssid = "buranic";
            String key = "h8sueb21";
            WifiConfiguration wifiConfig = new WifiConfiguration();
            wifiConfig.SSID = String.format("\"%s\"", ssid);
            wifiConfig.preSharedKey = String.format("\"%s\"", key);

            WifiManager wifiManager = (WifiManager) getSystemService(WIFI_SERVICE);
//remember id
            int netId = wifiManager.addNetwork(wifiConfig);
            wifiManager.disconnect();
            wifiManager.enableNetwork(netId, true);
            wifiManager.reconnect();
        }
    }
}
